@@ -1,4 +1,4 @@
-/* $OpenBSD: rsautl.c,v 1.23 2014/07/12 17:54:31 jsing Exp $ */
+/* $OpenBSD: rsautl.c,v 1.2 2014/08/28 14:23:52 jsing Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2000.
  */
@@ -107,8 +107,6 @@ rsautl_main(int argc, char **argv)
 	argc--;
 	argv++;
 
-	ERR_load_crypto_strings();
-	OpenSSL_add_all_algorithms();
 	pad = RSA_PKCS1_PADDING;
 
 	while (argc >= 1) {
