@@ -1,4 +1,4 @@
-/* $OpenBSD: s_server.c,v 1.1 2014/08/26 17:47:25 jsing Exp $ */
+/* $OpenBSD: s_server.c,v 1.6 2014/11/06 14:50:12 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -148,7 +148,6 @@
 
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#include <sys/select.h>
 #include <sys/socket.h>
 
 #include <assert.h>
@@ -158,6 +157,7 @@
 #include <limits.h>
 #include <string.h>
 #include <unistd.h>
+#include <poll.h>
 
 #include "apps.h"
 
@@ -166,7 +166,6 @@
 #include <openssl/lhash.h>
 #include <openssl/ocsp.h>
 #include <openssl/pem.h>
-#include <openssl/rand.h>
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
 
@@ -179,7 +178,6 @@
 #include "s_apps.h"
 #include "timeouts.h"
 
-static RSA *tmp_rsa_cb(SSL * s, int is_export, int keylength);
 static int sv_body(char *hostname, int s, unsigned char *context);
 static int www_body(char *hostname, int s, unsigned char *context);
 static void close_accept_socket(void);
@@ -191,40 +189,10 @@ generate_session_id(const SSL * ssl, unsigned char *id,
     unsigned int *id_len);
 #ifndef OPENSSL_NO_DH
 static DH *load_dh_param(const char *dhfile);
-static DH *get_dh512(void);
 #endif
 
 static void s_server_init(void);
 
-#ifndef OPENSSL_NO_DH
-static unsigned char dh512_p[] = {
-	0xDA, 0x58, 0x3C, 0x16, 0xD9, 0x85, 0x22, 0x89, 0xD0, 0xE4, 0xAF, 0x75,
-	0x6F, 0x4C, 0xCA, 0x92, 0xDD, 0x4B, 0xE5, 0x33, 0xB8, 0x04, 0xFB, 0x0F,
-	0xED, 0x94, 0xEF, 0x9C, 0x8A, 0x44, 0x03, 0xED, 0x57, 0x46, 0x50, 0xD3,
-	0x69, 0x99, 0xDB, 0x29, 0xD7, 0x76, 0x27, 0x6B, 0xA2, 0xD3, 0xD4, 0x12,
-	0xE2, 0x18, 0xF4, 0xDD, 0x1E, 0x08, 0x4C, 0xF6, 0xD8, 0x00, 0x3E, 0x7C,
-	0x47, 0x74, 0xE8, 0x33,
-};
-static unsigned char dh512_g[] = {
-	0x02,
-};
-
-static DH *
-get_dh512(void)
-{
-	DH *dh = NULL;
-
-	if ((dh = DH_new()) == NULL)
-		return (NULL);
-	dh->p = BN_bin2bn(dh512_p, sizeof(dh512_p), NULL);
-	dh->g = BN_bin2bn(dh512_g, sizeof(dh512_g), NULL);
-	if ((dh->p == NULL) || (dh->g == NULL))
-		return (NULL);
-	return (dh);
-}
-#endif
-
-
 /* static int load_CA(SSL_CTX *ctx, char *file);*/
 
 #define BUFSIZZ	16*1024
@@ -232,9 +200,7 @@ static int bufsize = BUFSIZZ;
 static int accept_socket = -1;
 
 #define TEST_CERT	"server.pem"
-#ifndef OPENSSL_NO_TLSEXT
 #define TEST_CERT2	"server2.pem"
-#endif
 
 extern int verify_depth, verify_return_error;
 
@@ -242,26 +208,20 @@ static char *cipher = NULL;
 static int s_server_verify = SSL_VERIFY_NONE;
 static int s_server_session_id_context = 1;	/* anything will do */
 static const char *s_cert_file = TEST_CERT, *s_key_file = NULL;
-#ifndef OPENSSL_NO_TLSEXT
 static const char *s_cert_file2 = TEST_CERT2, *s_key_file2 = NULL;
-#endif
 static char *s_dcert_file = NULL, *s_dkey_file = NULL;
 static int s_nbio = 0;
 static int s_nbio_test = 0;
 int s_crlf = 0;
 static SSL_CTX *ctx = NULL;
-#ifndef OPENSSL_NO_TLSEXT
 static SSL_CTX *ctx2 = NULL;
-#endif
 static int www = 0;
 
 static BIO *bio_s_out = NULL;
 static int s_debug = 0;
-#ifndef OPENSSL_NO_TLSEXT
 static int s_tlsextdebug = 0;
 static int s_tlsextstatus = 0;
 static int cert_status_cb(SSL * s, void *arg);
-#endif
 static int s_msg = 0;
 static int s_quiet = 0;
 
@@ -293,11 +253,9 @@ s_server_init(void)
 	s_dkey_file = NULL;
 	s_cert_file = TEST_CERT;
 	s_key_file = NULL;
-#ifndef OPENSSL_NO_TLSEXT
 	s_cert_file2 = TEST_CERT2;
 	s_key_file2 = NULL;
 	ctx2 = NULL;
-#endif
 	s_nbio = 0;
 	s_nbio_test = 0;
 	ctx = NULL;
@@ -356,7 +314,6 @@ sv_usage(void)
 	BIO_printf(bio_err, " -cipher arg   - play with 'openssl ciphers' to see what goes here\n");
 	BIO_printf(bio_err, " -serverpref   - Use server's cipher preferences\n");
 	BIO_printf(bio_err, " -quiet        - Inhibit printing of session and certificate information\n");
-	BIO_printf(bio_err, " -no_tmp_rsa   - Do not generate a tmp RSA key\n");
 	BIO_printf(bio_err, " -ssl3         - Just talk SSLv3\n");
 	BIO_printf(bio_err, " -tls1_2       - Just talk TLSv1.2\n");
 	BIO_printf(bio_err, " -tls1_1       - Just talk TLSv1.1\n");
@@ -383,7 +340,6 @@ sv_usage(void)
 	BIO_printf(bio_err, " -engine id    - Initialise and use the specified engine\n");
 #endif
 	BIO_printf(bio_err, " -id_prefix arg - Generate SSL/TLS session IDs prefixed by 'arg'\n");
-#ifndef OPENSSL_NO_TLSEXT
 	BIO_printf(bio_err, " -servername host - servername for HostName TLS extension\n");
 	BIO_printf(bio_err, " -servername_fatal - on mismatch send fatal alert (default warning alert)\n");
 	BIO_printf(bio_err, " -cert2 arg    - certificate file to use for servername\n");
@@ -397,7 +353,6 @@ sv_usage(void)
 #endif
 #ifndef OPENSSL_NO_SRTP
 	BIO_printf(bio_err, " -use_srtp profiles - Offer SRTP key management with a colon-separated profile list\n");
-#endif
 #endif
 	BIO_printf(bio_err, " -keymatexport label   - Export keying material using label\n");
 	BIO_printf(bio_err, " -keymatexportlen len  - Export len bytes of keying material (default 20)\n");
@@ -406,7 +361,6 @@ sv_usage(void)
 static int local_argc = 0;
 static char **local_argv;
 
-#ifndef OPENSSL_NO_TLSEXT
 
 /* This is a context that we pass to callbacks */
 typedef struct tlsextctx_st {
@@ -591,7 +545,6 @@ next_proto_cb(SSL * s, const unsigned char **data, unsigned int *len, void *arg)
 #endif				/* ndef OPENSSL_NO_NEXTPROTONEG */
 
 
-#endif
 
 int s_server_main(int, char **);
 
@@ -612,7 +565,7 @@ s_server_main(int argc, char *argv[])
 	int badop = 0, bugs = 0;
 	int ret = 1;
 	int off = 0;
-	int no_tmp_rsa = 0, no_dhe = 0, no_ecdhe = 0, nocert = 0;
+	int no_dhe = 0, no_ecdhe = 0, nocert = 0;
 	int state = 0;
 	const SSL_METHOD *meth = NULL;
 	int socket_type = SOCK_STREAM;
@@ -625,14 +578,12 @@ s_server_main(int argc, char *argv[])
 	EVP_PKEY *s_key = NULL, *s_dkey = NULL;
 	int no_cache = 0;
 	const char *errstr = NULL;
-#ifndef OPENSSL_NO_TLSEXT
 	EVP_PKEY *s_key2 = NULL;
 	X509 *s_cert2 = NULL;
 	tlsextctx tlsextcbp = {NULL, NULL, SSL_TLSEXT_ERR_ALERT_WARNING};
 #ifndef OPENSSL_NO_NEXTPROTONEG
 	const char *next_proto_neg_in = NULL;
 	tlsextnextprotoctx next_proto;
-#endif
 #endif
 	meth = SSLv23_server_method();
 
@@ -762,7 +713,6 @@ s_server_main(int argc, char *argv[])
 		} else if (strcmp(*argv, "-debug") == 0) {
 			s_debug = 1;
 		}
-#ifndef OPENSSL_NO_TLSEXT
 		else if (strcmp(*argv, "-tlsextdebug") == 0)
 			s_tlsextdebug = 1;
 		else if (strcmp(*argv, "-status") == 0)
@@ -790,7 +740,6 @@ s_server_main(int argc, char *argv[])
 				goto bad;
 			}
 		}
-#endif
 		else if (strcmp(*argv, "-msg") == 0) {
 			s_msg = 1;
 		} else if (strcmp(*argv, "-hack") == 0) {
@@ -804,13 +753,12 @@ s_server_main(int argc, char *argv[])
 		} else if (strcmp(*argv, "-bugs") == 0) {
 			bugs = 1;
 		} else if (strcmp(*argv, "-no_tmp_rsa") == 0) {
-			no_tmp_rsa = 1;
+			/* No-op. */
 		} else if (strcmp(*argv, "-no_dhe") == 0) {
 			no_dhe = 1;
 		} else if (strcmp(*argv, "-no_ecdhe") == 0) {
 			no_ecdhe = 1;
-		}
-		else if (strcmp(*argv, "-www") == 0) {
+		} else if (strcmp(*argv, "-www") == 0) {
 			www = 1;
 		} else if (strcmp(*argv, "-WWW") == 0) {
 			www = 2;
@@ -829,11 +777,9 @@ s_server_main(int argc, char *argv[])
 		} else if (strcmp(*argv, "-no_comp") == 0) {
 			off |= SSL_OP_NO_COMPRESSION;
 		}
-#ifndef OPENSSL_NO_TLSEXT
 		else if (strcmp(*argv, "-no_ticket") == 0) {
 			off |= SSL_OP_NO_TICKET;
 		}
-#endif
 		else if (strcmp(*argv, "-ssl3") == 0) {
 			meth = SSLv3_server_method();
 		} else if (strcmp(*argv, "-tls1") == 0) {
@@ -870,7 +816,6 @@ s_server_main(int argc, char *argv[])
 			engine_id = *(++argv);
 		}
 #endif
-#ifndef OPENSSL_NO_TLSEXT
 		else if (strcmp(*argv, "-servername") == 0) {
 			if (--argc < 1)
 				goto bad;
@@ -893,7 +838,6 @@ s_server_main(int argc, char *argv[])
 			next_proto_neg_in = *(++argv);
 		}
 #endif
-#endif
 #ifndef OPENSSL_NO_SRTP
 		else if (strcmp(*argv, "-use_srtp") == 0) {
 			if (--argc < 1)
@@ -939,10 +883,8 @@ bad:
 	}
 	if (s_key_file == NULL)
 		s_key_file = s_cert_file;
-#ifndef OPENSSL_NO_TLSEXT
 	if (s_key_file2 == NULL)
 		s_key_file2 = s_cert_file2;
-#endif
 
 	if (nocert == 0) {
 		s_key = load_key(bio_err, s_key_file, s_key_format, 0, pass, e,
@@ -958,7 +900,6 @@ bad:
 			ERR_print_errors(bio_err);
 			goto end;
 		}
-#ifndef OPENSSL_NO_TLSEXT
 		if (tlsextcbp.servername) {
 			s_key2 = load_key(bio_err, s_key_file2, s_key_format, 0, pass, e,
 			    "second server certificate private key file");
@@ -974,7 +915,6 @@ bad:
 				goto end;
 			}
 		}
-#endif
 	}
 #if !defined(OPENSSL_NO_TLSEXT) && !defined(OPENSSL_NO_NEXTPROTONEG)
 	if (next_proto_neg_in) {
@@ -1023,10 +963,8 @@ bad:
 		s_key_file = NULL;
 		s_dcert_file = NULL;
 		s_dkey_file = NULL;
-#ifndef OPENSSL_NO_TLSEXT
 		s_cert_file2 = NULL;
 		s_key_file2 = NULL;
-#endif
 	}
 	ctx = SSL_CTX_new(meth);
 	if (ctx == NULL) {
@@ -1082,7 +1020,6 @@ bad:
 	if (vpm)
 		SSL_CTX_set1_param(ctx, vpm);
 
-#ifndef OPENSSL_NO_TLSEXT
 	if (s_cert2) {
 		ctx2 = SSL_CTX_new(meth);
 		if (ctx2 == NULL) {
@@ -1139,7 +1076,6 @@ bad:
 	if (next_proto.data)
 		SSL_CTX_set_next_protos_advertised_cb(ctx, next_proto_cb, &next_proto);
 #endif
-#endif
 
 #ifndef OPENSSL_NO_DH
 	if (!no_dhe) {
@@ -1150,16 +1086,22 @@ bad:
 		else if (s_cert_file)
 			dh = load_dh_param(s_cert_file);
 
-		if (dh != NULL) {
+		if (dh != NULL)
 			BIO_printf(bio_s_out, "Setting temp DH parameters\n");
-		} else {
-			BIO_printf(bio_s_out, "Using default temp DH parameters\n");
-			dh = get_dh512();
-		}
+		else
+			BIO_printf(bio_s_out, "Using auto DH parameters\n");
 		(void) BIO_flush(bio_s_out);
 
-		SSL_CTX_set_tmp_dh(ctx, dh);
-#ifndef OPENSSL_NO_TLSEXT
+		if (dh == NULL)
+			SSL_CTX_set_dh_auto(ctx, 1);
+		else if (!SSL_CTX_set_tmp_dh(ctx, dh)) {
+			BIO_printf(bio_err,
+			    "Error setting temp DH parameters\n");
+			ERR_print_errors(bio_err);
+			DH_free(dh);
+			goto end;
+		}
+
 		if (ctx2) {
 			if (!dhfile) {
 				DH *dh2 = load_dh_param(s_cert_file2);
@@ -1171,9 +1113,16 @@ bad:
 					dh = dh2;
 				}
 			}
-			SSL_CTX_set_tmp_dh(ctx2, dh);
+			if (dh == NULL)
+				SSL_CTX_set_dh_auto(ctx2, 1);
+			else if (!SSL_CTX_set_tmp_dh(ctx2, dh)) {
+				BIO_printf(bio_err,
+				    "Error setting temp DH parameters\n");
+				ERR_print_errors(bio_err);
+				DH_free(dh);
+				goto end;
+			}
 		}
-#endif
 		DH_free(dh);
 	}
 #endif
@@ -1209,31 +1158,19 @@ bad:
 		(void) BIO_flush(bio_s_out);
 
 		SSL_CTX_set_tmp_ecdh(ctx, ecdh);
-#ifndef OPENSSL_NO_TLSEXT
 		if (ctx2)
 			SSL_CTX_set_tmp_ecdh(ctx2, ecdh);
-#endif
 		EC_KEY_free(ecdh);
 	}
 
 	if (!set_cert_key_stuff(ctx, s_cert, s_key))
 		goto end;
-#ifndef OPENSSL_NO_TLSEXT
 	if (ctx2 && !set_cert_key_stuff(ctx2, s_cert2, s_key2))
 		goto end;
-#endif
 	if (s_dcert != NULL) {
 		if (!set_cert_key_stuff(ctx, s_dcert, s_dkey))
 			goto end;
 	}
-	if (!no_tmp_rsa) {
-		SSL_CTX_set_tmp_rsa_callback(ctx, tmp_rsa_cb);
-#ifndef OPENSSL_NO_TLSEXT
-		if (ctx2)
-			SSL_CTX_set_tmp_rsa_callback(ctx2, tmp_rsa_cb);
-#endif
-	}
-
 
 	if (cipher != NULL) {
 		if (!SSL_CTX_set_cipher_list(ctx, cipher)) {
@@ -1241,13 +1178,11 @@ bad:
 			ERR_print_errors(bio_err);
 			goto end;
 		}
-#ifndef OPENSSL_NO_TLSEXT
 		if (ctx2 && !SSL_CTX_set_cipher_list(ctx2, cipher)) {
 			BIO_printf(bio_err, "error setting cipher list\n");
 			ERR_print_errors(bio_err);
 			goto end;
 		}
-#endif
 	}
 	SSL_CTX_set_verify(ctx, s_server_verify, verify_callback);
 	SSL_CTX_set_session_id_context(ctx, (void *) &s_server_session_id_context,
@@ -1257,7 +1192,6 @@ bad:
 	SSL_CTX_set_cookie_generate_cb(ctx, generate_cookie_callback);
 	SSL_CTX_set_cookie_verify_cb(ctx, verify_cookie_callback);
 
-#ifndef OPENSSL_NO_TLSEXT
 	if (ctx2) {
 		SSL_CTX_set_verify(ctx2, s_server_verify, verify_callback);
 		SSL_CTX_set_session_id_context(ctx2, (void *) &s_server_session_id_context,
@@ -1269,14 +1203,11 @@ bad:
 		SSL_CTX_set_tlsext_servername_callback(ctx, ssl_servername_cb);
 		SSL_CTX_set_tlsext_servername_arg(ctx, &tlsextcbp);
 	}
-#endif
 
 	if (CAfile != NULL) {
 		SSL_CTX_set_client_CA_list(ctx, SSL_load_client_CA_file(CAfile));
-#ifndef OPENSSL_NO_TLSEXT
 		if (ctx2)
 			SSL_CTX_set_client_CA_list(ctx2, SSL_load_client_CA_file(CAfile));
-#endif
 	}
 	BIO_printf(bio_s_out, "ACCEPT\n");
 	(void) BIO_flush(bio_s_out);
@@ -1301,7 +1232,6 @@ end:
 	free(dpass);
 	if (vpm)
 		X509_VERIFY_PARAM_free(vpm);
-#ifndef OPENSSL_NO_TLSEXT
 	free(tlscstatp.host);
 	free(tlscstatp.port);
 	free(tlscstatp.path);
@@ -1311,7 +1241,6 @@ end:
 		X509_free(s_cert2);
 	if (s_key2)
 		EVP_PKEY_free(s_key2);
-#endif
 	if (bio_s_out != NULL) {
 		BIO_free(bio_s_out);
 		bio_s_out = NULL;
@@ -1350,14 +1279,12 @@ static int
 sv_body(char *hostname, int s, unsigned char *context)
 {
 	char *buf = NULL;
-	fd_set readfds;
-	int ret = 1, width;
+	int ret = 1;
 	int k, i;
 	unsigned long l;
 	SSL *con = NULL;
 	BIO *sbio;
 	struct timeval timeout;
-	struct timeval *timeoutp;
 
 	if ((buf = malloc(bufsize)) == NULL) {
 		BIO_printf(bio_err, "out of memory\n");
@@ -1372,7 +1299,6 @@ sv_body(char *hostname, int s, unsigned char *context)
 
 	if (con == NULL) {
 		con = SSL_new(ctx);
-#ifndef OPENSSL_NO_TLSEXT
 		if (s_tlsextdebug) {
 			SSL_set_tlsext_debug_callback(con, tlsext_cb);
 			SSL_set_tlsext_debug_arg(con, bio_s_out);
@@ -1382,7 +1308,6 @@ sv_body(char *hostname, int s, unsigned char *context)
 			tlscstatp.err = bio_err;
 			SSL_CTX_set_tlsext_status_arg(ctx, &tlscstatp);
 		}
-#endif
 		if (context)
 			SSL_set_session_id_context(con, context,
 			    strlen((char *) context));
@@ -1434,42 +1359,50 @@ sv_body(char *hostname, int s, unsigned char *context)
 		SSL_set_msg_callback(con, msg_cb);
 		SSL_set_msg_callback_arg(con, bio_s_out);
 	}
-#ifndef OPENSSL_NO_TLSEXT
 	if (s_tlsextdebug) {
 		SSL_set_tlsext_debug_callback(con, tlsext_cb);
 		SSL_set_tlsext_debug_arg(con, bio_s_out);
 	}
-#endif
 
-	width = s + 1;
 	for (;;) {
 		int read_from_terminal;
 		int read_from_sslcon;
+		struct pollfd pfd[2];
+		int ptimeout;
 
 		read_from_terminal = 0;
 		read_from_sslcon = SSL_pending(con);
 
 		if (!read_from_sslcon) {
-			FD_ZERO(&readfds);
-			FD_SET(fileno(stdin), &readfds);
-			FD_SET(s, &readfds);
+			pfd[0].fd = fileno(stdin);
+			pfd[0].events = POLLIN;
+			pfd[1].fd = s;
+			pfd[1].events = POLLIN;
+
 			if ((SSL_version(con) == DTLS1_VERSION) &&
 			    DTLSv1_get_timeout(con, &timeout))
-				timeoutp = &timeout;
+				ptimeout = timeout.tv_sec * 1000 +
+				    timeout.tv_usec / 1000;
 			else
-				timeoutp = NULL;
+				ptimeout = -1;
 
-			i = select(width, &readfds, NULL, NULL, timeoutp);
+			i = poll(pfd, 2, ptimeout);
 
 			if ((SSL_version(con) == DTLS1_VERSION) && DTLSv1_handle_timeout(con) > 0) {
 				BIO_printf(bio_err, "TIMEOUT occured\n");
 			}
 			if (i <= 0)
 				continue;
-			if (FD_ISSET(fileno(stdin), &readfds))
+			if (pfd[0].revents) {
+				if ((pfd[0].revents & (POLLERR|POLLNVAL)))
+					continue;
 				read_from_terminal = 1;
-			if (FD_ISSET(s, &readfds))
+			}
+			if (pfd[1].revents) {
+				if ((pfd[1].revents & (POLLERR|POLLNVAL)))
+					continue;
 				read_from_sslcon = 1;
+			}
 		}
 		if (read_from_terminal) {
 			if (s_crlf) {
@@ -1808,12 +1741,10 @@ www_body(char *hostname, int s, unsigned char *context)
 
 	if ((con = SSL_new(ctx)) == NULL)
 		goto err;
-#ifndef OPENSSL_NO_TLSEXT
 	if (s_tlsextdebug) {
 		SSL_set_tlsext_debug_callback(con, tlsext_cb);
 		SSL_set_tlsext_debug_arg(con, bio_s_out);
 	}
-#endif
 	if (context)
 		SSL_set_session_id_context(con, context,
 		    strlen((char *) context));
@@ -2095,34 +2026,6 @@ err:
 	return (ret);
 }
 
-static RSA *
-tmp_rsa_cb(SSL * s, int is_export, int keylength)
-{
-	BIGNUM *bn = NULL;
-	static RSA *rsa_tmp = NULL;
-
-	if (!rsa_tmp && ((bn = BN_new()) == NULL))
-		BIO_printf(bio_err, "Allocation error in generating RSA key\n");
-	if (!rsa_tmp && bn) {
-		if (!s_quiet) {
-			BIO_printf(bio_err, "Generating temp (%d bit) RSA key...", keylength);
-			(void) BIO_flush(bio_err);
-		}
-		if (!BN_set_word(bn, RSA_F4) || ((rsa_tmp = RSA_new()) == NULL) ||
-		    !RSA_generate_key_ex(rsa_tmp, keylength, bn, NULL)) {
-			if (rsa_tmp)
-				RSA_free(rsa_tmp);
-			rsa_tmp = NULL;
-		}
-		if (!s_quiet) {
-			BIO_printf(bio_err, "\n");
-			(void) BIO_flush(bio_err);
-		}
-		BN_free(bn);
-	}
-	return (rsa_tmp);
-}
-
 #define MAX_SESSION_ID_ATTEMPTS 10
 static int
 generate_session_id(const SSL * ssl, unsigned char *id,
@@ -2130,7 +2033,7 @@ generate_session_id(const SSL * ssl, unsigned char *id,
 {
 	unsigned int count = 0;
 	do {
-		RAND_pseudo_bytes(id, *id_len);
+		arc4random_buf(id, *id_len);
 		/*
 		 * Prefix the session_id with the required prefix. NB: If our
 		 * prefix is too long, clip it - but there will be worse
