@@ -1,4 +1,12 @@
-#if defined(__linux__)
+#ifndef LIBCRYPTOCOMPAT_ARC4RANDOM_H
+#define LIBCRYPTOCOMPAT_ARC4RANDOM_H
+
+#include <sys/param.h>
+
+#if defined(__FreeBSD__)
+#include "arc4random_freebsd.h"
+
+#elif defined(__linux__)
 #include "arc4random_linux.h"
 
 #elif defined(__APPLE__)
@@ -7,7 +15,7 @@
 #elif defined(__sun)
 #include "arc4random_solaris.h"
 
-#elif defined(__WIN32)
+#elif defined(_WIN32)
 #include "arc4random_win.h"
 
 #else
@@ -15,3 +23,4 @@
 
 #endif
 
+#endif
