@@ -63,6 +63,7 @@
 #include <netinet/in.h>
 
 #include <stdio.h>
+
 #include <openssl/objects.h>
 
 #include "pqueue.h"
@@ -449,9 +450,6 @@ dtls1_listen(SSL *s, struct sockaddr *client)
 {
 	int ret;
 
-	/* Ensure there is no state left over from a previous invocation */
-	SSL_clear(s);
-
 	SSL_set_options(s, SSL_OP_COOKIE_EXCHANGE);
 	s->d1->listen = 1;
 
