@@ -1,4 +1,4 @@
-/* $OpenBSD: d1_pkt.c,v 1.32 2014/07/10 08:51:14 tedu Exp $ */
+/* $OpenBSD: d1_pkt.c,v 1.36 2014/10/22 15:29:31 jsing Exp $ */
 /* 
  * DTLS implementation written by Nagendra Modadugu
  * (nagendra@cs.stanford.edu) for the OpenSSL project 2005.  
@@ -113,13 +113,15 @@
  * [including the GNU Public Licence.]
  */
 
-#include <stdio.h>
-#include <errno.h>
 #include <machine/endian.h>
+
+#include <errno.h>
+#include <stdio.h>
+
 #include "ssl_locl.h"
-#include <openssl/evp.h>
+
 #include <openssl/buffer.h>
-#include <openssl/rand.h>
+#include <openssl/evp.h>
 
 #include "pqueue.h"
 
@@ -587,7 +589,7 @@ again:
 		 * would be dropped unnecessarily.
 		 */
 		if (!(s->d1->listen && rr->type == SSL3_RT_HANDSHAKE &&
-		    *p == SSL3_MT_CLIENT_HELLO) &&
+		    p != NULL && *p == SSL3_MT_CLIENT_HELLO) &&
 		    !dtls1_record_replay_check(s, bitmap)) {
 			rr->length = 0;
 			s->packet_length=0; /* dump this record */
@@ -1380,7 +1382,7 @@ do_dtls1_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 	/* ssl3_enc can only have an error on read */
 	if (bs)	/* bs != 0 in case of CBC */
 	{
-		RAND_pseudo_bytes(p, bs);
+		arc4random_buf(p, bs);
 		/* master IV and last CBC residue stand for
 		 * the rest of randomness */
 		wr->length += bs;
