@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_lib.c,v 1.85 2014/10/03 13:58:18 jsing Exp $ */
+/* $OpenBSD: ssl_lib.c,v 1.89 2014/10/31 15:25:55 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -141,13 +141,15 @@
  */
 
 #include <stdio.h>
+
 #include "ssl_locl.h"
-#include <openssl/objects.h>
-#include <openssl/lhash.h>
-#include <openssl/x509v3.h>
-#include <openssl/rand.h>
-#include <openssl/ocsp.h>
+
 #include <openssl/dh.h>
+#include <openssl/lhash.h>
+#include <openssl/objects.h>
+#include <openssl/ocsp.h>
+#include <openssl/x509v3.h>
+
 #ifndef OPENSSL_NO_ENGINE
 #include <openssl/engine.h>
 #endif
@@ -1786,11 +1788,11 @@ SSL_CTX_new(const SSL_METHOD *meth)
 
 	ret->tlsext_servername_callback = 0;
 	ret->tlsext_servername_arg = NULL;
+
 	/* Setup RFC4507 ticket keys */
-	if ((RAND_pseudo_bytes(ret->tlsext_tick_key_name, 16) <= 0)
-	    || (RAND_bytes(ret->tlsext_tick_hmac_key, 16) <= 0)
-	    || (RAND_bytes(ret->tlsext_tick_aes_key, 16) <= 0))
-		ret->options |= SSL_OP_NO_TICKET;
+	arc4random_buf(ret->tlsext_tick_key_name, 16);
+	arc4random_buf(ret->tlsext_tick_hmac_key, 16);
+	arc4random_buf(ret->tlsext_tick_aes_key, 16);
 
 	ret->tlsext_status_cb = 0;
 	ret->tlsext_status_arg = NULL;
@@ -1932,7 +1934,7 @@ void
 ssl_set_cert_masks(CERT *c, const SSL_CIPHER *cipher)
 {
 	CERT_PKEY	*cpk;
-	int		 rsa_enc, rsa_tmp, rsa_sign, dh_tmp, dsa_sign;
+	int		 rsa_enc, rsa_sign, dh_tmp, dsa_sign;
 	unsigned long	 mask_k, mask_a;
 	int		 have_ecc_cert, ecdh_ok, ecdsa_ok;
 	int		 have_ecdh_tmp;
@@ -1943,8 +1945,8 @@ ssl_set_cert_masks(CERT *c, const SSL_CIPHER *cipher)
 	if (c == NULL)
 		return;
 
-	rsa_tmp = (c->rsa_tmp != NULL || c->rsa_tmp_cb != NULL);
-	dh_tmp = (c->dh_tmp != NULL || c->dh_tmp_cb != NULL);
+	dh_tmp = (c->dh_tmp != NULL || c->dh_tmp_cb != NULL ||
+	    c->dh_tmp_auto != 0);
 
 	have_ecdh_tmp = (c->ecdh_tmp != NULL || c->ecdh_tmp_cb != NULL ||
 	    c->ecdh_tmp_auto != 0);
@@ -1971,7 +1973,7 @@ ssl_set_cert_masks(CERT *c, const SSL_CIPHER *cipher)
 		mask_a |= SSL_aGOST94;
 	}
 
-	if (rsa_enc || (rsa_tmp && rsa_sign))
+	if (rsa_enc)
 		mask_k|=SSL_kRSA;
 
 	if (dh_tmp)
@@ -2178,6 +2180,54 @@ ssl_get_sign_pkey(SSL *s, const SSL_CIPHER *cipher, const EVP_MD **pmd)
 	return (c->pkeys[idx].privatekey);
 }
 
+DH *
+ssl_get_auto_dh(SSL *s)
+{
+	CERT_PKEY *cpk;
+	int keylen;
+	DH *dhp;
+
+	if (s->cert->dh_tmp_auto == 2) {
+		keylen = 1024;
+	} else if (s->s3->tmp.new_cipher->algorithm_auth & SSL_aNULL) {
+		keylen = 1024;
+		if (s->s3->tmp.new_cipher->strength_bits == 256)
+			keylen = 3072;
+	} else {
+		if ((cpk = ssl_get_server_send_pkey(s)) == NULL)
+			return (NULL);
+		if (cpk->privatekey == NULL || cpk->privatekey->pkey.dh == NULL)
+			return (NULL);
+		keylen = EVP_PKEY_bits(cpk->privatekey);
+	}
+
+	if ((dhp = DH_new()) == NULL)
+		return (NULL);
+
+	dhp->g = BN_new();
+	if (dhp->g != NULL)
+		BN_set_word(dhp->g, 2);
+
+	if (keylen >= 8192)
+		dhp->p = get_rfc3526_prime_8192(NULL);
+	else if (keylen >= 4096)
+		dhp->p = get_rfc3526_prime_4096(NULL);
+	else if (keylen >= 3072)
+		dhp->p = get_rfc3526_prime_3072(NULL);
+	else if (keylen >= 2048)
+		dhp->p = get_rfc3526_prime_2048(NULL);
+	else if (keylen >= 1536)
+		dhp->p = get_rfc3526_prime_1536(NULL);
+	else
+		dhp->p = get_rfc2409_prime_1024(NULL);
+
+	if (dhp->p == NULL || dhp->g == NULL) {
+		DH_free(dhp);
+		return (NULL);
+	}
+	return (dhp);
+}
+
 void
 ssl_update_cache(SSL *s, int mode)
 {
