@@ -431,7 +431,7 @@ main(int argc, char *argv[])
 	int badop = 0;
 	int bio_pair = 0;
 	int force = 0;
-	int tls1 = 0, ssl2 = 0, ssl3 = 0, dtls1 = 0, ret = 1;
+	int tls1 = 0, ssl3 = 0, dtls1 = 0, ret = 1;
 	int client_auth = 0;
 	int server_auth = 0, i;
 	struct app_verify_arg app_verify_arg =
@@ -501,8 +501,6 @@ main(int argc, char *argv[])
 			no_ecdhe = 1;
 		else if (strcmp(*argv, "-dtls1") == 0)
 			dtls1 = 1;
-		else if (strcmp(*argv, "-ssl2") == 0)
-			ssl2 = 1;
 		else if (strcmp(*argv, "-ssl3") == 0)
 			ssl3 = 1;
 		else if (strcmp(*argv, "-tls1") == 0)
@@ -608,12 +606,12 @@ bad:
 		goto end;
 	}
 
-	if (!dtls1 && !ssl2 && !ssl3 && !tls1 &&
+	if (!dtls1 && !ssl3 && !tls1 &&
 	    number > 1 && !reuse && !force) {
 		fprintf(stderr,
 		    "This case cannot work.  Use -f to perform "
 		    "the test anyway (and\n-d to see what happens), "
-		    "or add one of -dtls1, -ssl2, -ssl3, -tls1, -reuse\n"
+		    "or add one of -dtls1, -ssl3, -tls1, -reuse\n"
 		    "to avoid protocol mismatch.\n");
 		exit(1);
 	}
